/*
	
	ColumnModel.cpp
	
	by Chris Page
	
*/

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef COLUMN_MODEL_H
#include "ColumnModel.h"
#endif


// These two constants are "soft". They are used to create BLists
// and tell the list how many items to allocate at a time. They
// are only used for efficiency and do not affect correct operation.

const int cFaceDownBlockSize = 6;
const int cFaceUpBlockSize = 13;


// Create an empty column.

ColumnModel::ColumnModel()
{
	DB_TRACE("ColumnModel::ColumnModel");
	
	this->faceDownCards = new BList(cFaceDownBlockSize);
	this->faceUpCards = new BList(cFaceUpBlockSize);
}


// Delete all the cards in the column.

ColumnModel::~ColumnModel()
{
	DB_TRACE("ColumnModel::~ColumnModel");
	
	while (!this->faceDownCards->IsEmpty())
		delete (CardModel*) this->faceDownCards->RemoveItem(0L);
	delete this->faceDownCards;
	while (!this->faceUpCards->IsEmpty())
		delete (CardModel*) this->faceUpCards->RemoveItem(0L);
	delete this->faceUpCards;
}


// Add a face-up card to the bottom of the column.

void ColumnModel::Add(CardModel *card)
{
	DB_TRACE("ColumnModel::Add");
	
	this->faceUpCards->AddItem(card);
}


// Add a list of face-up cards to the bottom of the column.

void ColumnModel::Add(BList *cards)
{
	DB_TRACE("ColumnModel::Add");
	
	this->faceUpCards->AddList(cards);
}


// Add a face-down card to the bottom of the column.

void ColumnModel::AddFaceDown(CardModel *card)
{
	DB_TRACE("ColumnModel::AddFaceDown");
	
	this->faceDownCards->AddItem(card);
}


// Remove the specified face-up card from the column. Returns a
// list of cards, containing the specified card and all the cards
// below it in the column. It is an error to attempt to remove a
// card that is not in the column.

BList *ColumnModel::Remove(CardModel *card)
{
	DB_TRACE("ColumnModel::Remove");
	
	BList *removedCards = new BList(cFaceUpBlockSize);
	long cardIndex = this->faceUpCards->IndexOf(card);
	
	while (TRUE)
	{
		CardModel *aCard
			= (CardModel*) this->faceUpCards->RemoveItem(cardIndex);
		if (aCard == NULL)
			break;
		removedCards->AddItem(aCard);
	}
	
	// Make sure the new bottom card is face-up.
	TurnBottomFaceUp();
	
	return removedCards;
}


// Return the bottom card in the column. This does not remove the
// card.

CardModel *ColumnModel::Bottom() const
{
	DB_TRACE("ColumnModel::Bottom");
	
	return (CardModel*) this->faceUpCards->LastItem();
}


// Return the face-up card at a specific index. This does not remove
// the card.

CardModel *ColumnModel::CardFaceUpAt(long index) const
{
	DB_TRACE("ColumnModel::CardFaceUpAt");
	
	return (CardModel*) this->faceUpCards->ItemAt(
		this->faceUpCards->CountItems() - 1 - index);
}


// Return TRUE if the specified card is in the column and it is
// face-up.

bool ColumnModel::HasFaceUp(CardModel *card) const
{
	DB_TRACE("ColumnModel::HasFaceUp");
	
	return this->faceUpCards->HasItem(card);
}


// Return TRUE if all the cards in the column are face-up.

bool ColumnModel::AllFaceUp() const
{
	DB_TRACE("ColumnModel::AllFaceUp");
	
	return this->faceDownCards->IsEmpty();
}


// Return TRUE if the column is empty (there are no cards).

bool ColumnModel::IsEmpty() const
{
	DB_TRACE("ColumnModel::IsEmpty");
	
	return this->faceDownCards->IsEmpty()
			&& this->faceUpCards->IsEmpty();
}


// Return the number of cards in the column (face-up and face-down).

long ColumnModel::Count() const
{
	DB_TRACE("ColumnModel::Count");
	
	return this->faceDownCards->CountItems()
			+ this->faceUpCards->CountItems();
}


// Return the number of face-down cards in the column.

long ColumnModel::CountFaceDown() const
{
	DB_TRACE("ColumnModel::CountFaceDown");
	
	return this->faceDownCards->CountItems();
}


// Return the number of face-up cards in the column.

long ColumnModel::CountFaceUp() const
{
	DB_TRACE("ColumnModel::CountFaceUp");
	
	return this->faceUpCards->CountItems();
}


// Turn the bottom card face-up, if it isn't already.

void ColumnModel::TurnBottomFaceUp()
{
	DB_TRACE("ColumnModel::TurnBottomFaceUp");
	
	if (CountFaceUp() == 0 && CountFaceDown() > 0)
	{
		CardModel *card = (CardModel*)
			this->faceDownCards->RemoveItem(CountFaceDown() - 1);
		Add(card);
	}
};
