/*
	
	DeckModel.cpp
	
	by Chris Page
	
*/

#ifndef __stdlib__
#include <stdlib.h>
#endif

#ifndef __time__
#include <time.h>
#endif

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef DECK_MODEL_H
#include "DeckModel.h"
#endif


// Create a deck with 52 sorted cards. The cards are grouped by
// suit, then sorted by rank. Note that the suits are unordered
// so their order within the deck is arbitrary.

DeckModel::DeckModel()
{
	DB_TRACE("DeckModel::DeckModel");
	
	// Create cards and add to deck.
	
	this->cards = new BList(52);
	this->returnedCards = new BList(52);
	
	for (CardModel::Suit suit = CardModel::cHearts;
			suit <= CardModel::cSpades;
			suit = (CardModel::Suit)((int) suit + 1))
		for (CardModel::Rank rank = 1; rank <= 13; rank++)
			this->cards->AddItem(new CardModel(suit, rank));
}


// Delete all the cards in the deck.

DeckModel::~DeckModel()
{
	DB_TRACE("DeckModel::~DeckModel");
	
	while (!this->cards->IsEmpty())
		delete (CardModel*) this->cards->RemoveItem(0L);
	delete this->cards;
	while (!this->returnedCards->IsEmpty())
		delete (CardModel*) this->returnedCards->RemoveItem(0L);
	delete this->returnedCards;
}


// Sort comparison function used to shuffle the deck. It returns a
// value to indicate the ordering of two items. However, instead of
// comparing them, this function returns a random value. This should
// cause the sorting function to randomize the items.

static int RandomCompare(const void*, const void*)
{
	DB_TRACE("DeckModel::RandomCompare");
	
	if (rand() > RAND_MAX / 2)
		return 1;
	
	return -1;
}


// Shuffle the cards in the deck. Returns self.

DeckModel *DeckModel::Shuffle()
{
	DB_TRACE("DeckModel::Shuffle");
	
	// Shuffling is accomplished by "sorting" the deck with a
	// randomizing comparison function.
	
	// NOTE: It might be interesting to attempt to simulate a
	// shuffle by cutting the deck and inserting cards repeatedly.
	
	// Be sure to re-seed rand() before shuffling.
	srand(clock());
	
	// Shuffle several times just to mix it up well.
	this->cards->SortItems(RandomCompare);
	this->cards->SortItems(RandomCompare);
	this->cards->SortItems(RandomCompare);
	
	return this;
}


// Deal a card from the top of the deck. The card is removed
// from the deck. If "dealThrough" is TRUE and there are no cards
// in the deck, Deal() will attempt to deal a card from the returned
// cards, if there are any. If no card could be dealt, NULL is
// returned.

CardModel *DeckModel::Deal(bool dealThrough)
{
	DB_TRACE("DeckModel::Deal");
	
	CardModel *card = (CardModel*) this->cards->RemoveItem(0L);
	if (card == NULL && dealThrough)
	{
		card = (CardModel*) this->returnedCards->RemoveItem(0L);
	}
	
	return card;
}


// Return a card to the deck. Returned cards are held separately
// from cards that have not been delt. When all cards have been
// delt, the returned cards may be placed back into the deck with
// DeckModel::Restart().

void DeckModel::Return(CardModel *card)
{
	DB_TRACE("DeckModel::Return");
	
	this->returnedCards->AddItem(card);
}


// Restart the deck by placing any returned cards at the bottom
// of the deck.

void DeckModel::Restart()
{
	DB_TRACE("DeckModel::Restart");
	
	this->cards->AddList(this->returnedCards);
	this->returnedCards->MakeEmpty();
}


// Return TRUE if the deck is empty.

bool DeckModel::IsEmpty() const
{
	DB_TRACE("DeckModel::IsEmpty");
	
	return this->cards->IsEmpty() && this->returnedCards->IsEmpty();
}


// Return TRUE if the deck has no cards to deal (if there are any
// cards, they are only in the returned cards).

bool DeckModel::IsCardsEmpty() const
{
	DB_TRACE("DeckModel::IsCardsEmpty");
	
	return this->cards->IsEmpty();
}


// Return the number of cards in the deck (not including returned
// cards).

long DeckModel::CountCards() const
{
	DB_TRACE("DeckModel::CountCards");
	
	return this->cards->CountItems();
}
