/*
	
	SolitaireModel.cpp
	
	by Chris Page
	
*/

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef DECK_MODEL_H
#include "DeckModel.h"
#endif

#ifndef HAND_MODEL_H
#include "HandModel.h"
#endif

#ifndef COLUMN_MODEL_H
#include "ColumnModel.h"
#endif

#ifndef FOUNDATION_MODEL_H
#include "FoundationModel.h"
#endif

#ifndef SOLITAIRE_MODEL_H
#include "SolitaireModel.h"
#endif


// Create a solitaire game. Create a deck and initialize the hand,
// columns, and foundations.

SolitaireModel::SolitaireModel()
{
	DB_TRACE("SolitaireModel::SolitaireModel");
	
	this->deck = new DeckModel;
	this->hand = new HandModel;
	for (long i = 0; i < cColumnCount; i++)
		this->columns[i] = new ColumnModel;
	for (long i = 0; i < cFoundationCount; i++)
		this->foundations[i] = new FoundationModel;
	
	// Deal the initial column cards.
	this->deck->Shuffle();
	for (long start = 0; start < cColumnCount; start++)
		for (long column = start; column < cColumnCount; column++)
		{
			CardModel *card = this->deck->Deal();
			if (column == start)
				this->columns[column]->Add(card);
			else
				this->columns[column]->AddFaceDown(card);
		}
}


// Delete all the game data.

SolitaireModel::~SolitaireModel()
{
	DB_TRACE("SolitaireModel::~SolitaireModel");
	
	delete this->deck;
	delete this->hand;
	for (long i = 0; i < cColumnCount; i++)
		delete this->columns[i];
	for (long i = 0; i < 4; i++)
		delete this->foundations[i];
}


// Return TRUE if the game has been won. This checks to see if
// the game is in a winning state. A winning state is when:
//
//		* All cards are face-up
//		* There are no cards in the hand
//
// The first rule also implies that the deck must be empty.

bool SolitaireModel::Won() const
{
	DB_TRACE("SolitaireModel::Won");
	
	if (this->deck->IsEmpty() && this->hand->IsEmpty())
	{
		for (long i = 0; i < cColumnCount; i++)
			if (!this->columns[i]->AllFaceUp())
				return FALSE;
		
		return TRUE;
	}
	
	return FALSE;
}


// Deal cards from the deck into the hand. If there are already
// cards in the hand, return them to the deck first.

void SolitaireModel::DealHand()
{
	DB_TRACE("SolitaireModel::DealHand");
	
	// Return the hand to the deck.
	
	while (this->hand->Count() > 0)
		this->deck->Return(this->hand->RemoveBottom());
	
	// If the deck has no more cards to deal, restart the deck,
	// else deal another hand.
	
	if (this->deck->IsCardsEmpty())
		this->deck->Restart();
	else
	{
		// A hand is three cards. If there are less than three cards
		// in the deck, the remaining cards are dealt.
		
		long cardCount = this->deck->CountCards();
		if (cardCount > HandModel::cCardCount)
			cardCount = HandModel::cCardCount;
		
		while (cardCount-- > 0)
			this->hand->Add(this->deck->Deal());
	}
}


// Return TRUE if the specified card can be moved from its
// current location. Cards that can be moved are:
//
//		* The top card of the hand
//		* Any face-up card in a column

bool SolitaireModel::IsMovable(CardModel *card) const
{
	DB_TRACE("SolitaireModel::IsMovable");
	
	if (this->hand->Top() == card)
		return TRUE;
	
	for (long i = 0; i < cColumnCount; i++)
		if (this->columns[i]->HasFaceUp(card))
			return TRUE;
	
	return FALSE;
}


// Return TRUE if the specified card can be moved from its
// current location to the bottom of the specified column.
// The move is valid when the card is movable and one of
// the following is true:
//
//		- The card is a King and the column is empty
//		- The card is not the same color as the bottom card
//		  in the column and it is one rank lower than the
//		  bottom card

bool SolitaireModel::IsMovable(CardModel *card,
							   ColumnModel *toColumn) const
{
	DB_TRACE("SolitaireModel::IsMovable");
	
	if (this->IsMovable(card))
	{
		if (toColumn->IsEmpty())
			if (card->rank == 13)
				return TRUE;
			else
				return FALSE;
		
		CardModel *columnCard = toColumn->Bottom();
		if (!card->IsSameColor(columnCard)
				&& card->rank == (columnCard->rank - 1))
			return TRUE;
	}
	
	return FALSE;
}


// Return TRUE if the specified card can be moved from its
// current location to the top of the specified foundation.
// The move is valid when the card is movable and one of
// the following is true:
//
//		- The card is an Ace and the foundation is empty
//		- The card is the same color as the top card
//		  in the foundation and it is one rank higher than
//		  the top card

bool SolitaireModel::IsMovable(CardModel *card,
							   FoundationModel *toFoundation) const
{
	DB_TRACE("SolitaireModel::IsMovable");
	
	if (this->IsMovable(card))
	{
		if (toFoundation->IsEmpty())
			if (card->rank == 1)
				return TRUE;
			else
				return FALSE;
		
		CardModel *foundationCard = toFoundation->Top();
		if (card->suit == foundationCard->suit
				&& card->rank == (foundationCard->rank + 1))
		{
			// If this card is from a column, make sure it is the
			// bottom card, and not some interior card.
			
			for (long i = 0; i < cColumnCount; i++)
				if (this->columns[i]->HasFaceUp(card))
					if (card != this->columns[i]->Bottom())
						return FALSE;
			
			return TRUE;
		}
	}
	
	return FALSE;
}


// Move a card to the specified column. It is an error to attempt
// an invalid move.

void SolitaireModel::Move(CardModel *card, ColumnModel *toColumn)
{
	DB_TRACE("SolitaireModel::Move");
	
	if (!this->IsMovable(card, toColumn))
		/* error */ return;
	
	// Find the current location of the card and move it to the
	// destination.
	
	if (this->hand->Top() == card)
	{
		(void) this->hand->Remove();
		toColumn->Add(card);
		
		// If the hand is now empty, deal a single card. Passing
		// TRUE to Deal() means that returned cards can also
		// be dealt.
		if (this->hand->IsEmpty() && !this->deck->IsEmpty())
		{
			this->hand->Add(this->deck->Deal(TRUE));
		}
	}
	else
	{
		// Find the card source.
		ColumnModel *sourceColumn = NULL;
		for (long i = 0; i < cColumnCount; i++)
		{
			if (this->columns[i]->HasFaceUp(card))
			{
				sourceColumn = this->columns[i];
				break;
			}
		}
		
		toColumn->Add(sourceColumn->Remove(card));
	}
}


// Move a card to the specified foundation. It is an error to attempt
// an invalid move.

void SolitaireModel::Move(CardModel *card,
						  FoundationModel *toFoundation)
{
	DB_TRACE("SolitaireModel::Move");
	
	if (!this->IsMovable(card, toFoundation))
		/* error */ return;
	
	// Find the current location of the card and move it to the
	// destination.
	
	if (this->hand->Top() == card)
	{
		(void) this->hand->Remove();
		toFoundation->Add(card);
		
		// If the hand is now empty, deal a single card. Passing
		// TRUE to Deal() means that returned cards can also
		// be dealt.
		if (this->hand->IsEmpty() && !this->deck->IsEmpty())
		{
			this->hand->Add(this->deck->Deal(TRUE));
		}
	}
	else
	{
		// Find the card source.
		ColumnModel *sourceColumn = NULL;
		for (long i = 0; i < cColumnCount; i++)
		{
			if (this->columns[i]->HasFaceUp(card))
			{
				sourceColumn = this->columns[i];
				break;
			}
		}
		
		BList *cardList = sourceColumn->Remove(card);
		CardModel *card = (CardModel*) cardList->FirstItem();
		toFoundation->Add(card);
	}
}
