/*
	
	CardView.cpp
	
	by Chris Page
	
*/

#ifndef _REGION_H
#include <Region.h>
#endif

#ifndef _VIEW_H
#include <View.h>
#endif

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif


CardView::CardView(BView* view, BRect frame, CardModel* card, bool faceUp)
{
	DB_TRACE("CardView::CardView");
	
	this->view = view;
	this->frame = frame;
	this->card = card;
	this->faceUp = faceUp;
}


// Create the view. This creates a card view that does not have a
// card model. Without a card model, the card front will appear blank.
// This is only used to draw either the back of the card, or a stack
// of cards, where the front will not be visible.

CardView::CardView(BView* view, BRect frame)
{
	DB_TRACE("CardView::CardView");
	
	this->view = view;
	this->frame = frame;
	this->card = NULL;
	this->faceUp = FALSE;
}


// Draw the front of the card.

void CardView::DrawFront()
{
	DB_TRACE("CardView::DrawFront");
	
	BRect aViewBounds = this->frame;
	
	this->view->SetHighColor(255, 255, 255);
	this->view->FillRoundRect(aViewBounds, cCardRadius, cCardRadius);
	
	// Draw the card front, if there is a card model.
	if (this->card != NULL)
	{
		long suitColorIndex;
		rgb_color suitColors[2][2] = {
			{{255, 0, 0},	{128, 0, 0}},		// Reds
			{{0, 0, 0},		{128, 128, 128}}	// Blacks
		};
		
		// Set the drawing color for the suit.
		if (this->card->suit == CardModel::cHearts
				|| this->card->suit == CardModel::cDiamonds)
			suitColorIndex = 0;
		else
			suitColorIndex = 1;
		
		this->view->SetHighColor(suitColors[suitColorIndex][0]);
		
		// Draw the rank and suit.
		{
			char* rankStrings[] = {
				"*", "A", "2", "3", "4", "5", "6",
				"7", "8", "9", "10", "J", "Q", "K"
			};
			char suitChars[] = { '', '', '', '' };
			
			char* aRankString = rankStrings[this->card->rank];
			char aSuitChar = suitChars[this->card->suit];
			
			// Draw rank and suit in left/top corner.
			
			this->view->SetFontName("Arial MT Bold");
			this->view->SetFontSize(16);
			this->view->DrawString(aRankString,
				aViewBounds.LeftTop() + BPoint(6, 16));
			
			this->view->SetSymbolSet("Symbol");
			this->view->SetFontName("Symbol MT");
			this->view->SetFontSize(20);
			this->view->DrawChar(' ');
			this->view->DrawChar(aSuitChar);
			this->view->SetSymbolSet("Macintosh");
			
			// Draw rank and suit on card face.
			
			this->view->SetHighColor(suitColors[suitColorIndex][1]);
			this->view->SetSymbolSet("Symbol");
			this->view->SetFontName("Symbol MT");
			this->view->SetFontSize(90);
			long aCharWidth = this->view->StringWidth(&aSuitChar, 1);
			this->view->DrawChar(aSuitChar, aViewBounds.LeftBottom()
				+ BPoint((this->frame.Width() - aCharWidth) / 2, -5));
			
			this->view->SetHighColor(suitColors[suitColorIndex][0]);
			this->view->SetSymbolSet("Macintosh");
			this->view->SetFontName("Arial MT Bold Italic");
			this->view->SetFontSize(48);
		// CBP 2/1/96: For some reason, StringWidth() crashes. I think
		// it may be a corrupt font (or, of course, a system software
		// bug).
		//	long aStringWidth = this->view->StringWidth(aRankString);
			long aStringWidth = 32 + (aRankString[1] != 0 ? 32 : 0);
			this->view->DrawString(aRankString, aViewBounds.LeftBottom()
				+ BPoint((this->frame.Width() - aStringWidth) / 2, -15));
		}
	}
	
	// Draw the edges.
	this->view->SetHighColor(0, 0, 0);
	this->view->StrokeRoundRect(aViewBounds, cCardRadius, cCardRadius);
}


// Draw the back of the card.

void CardView::DrawBack()
{
	DB_TRACE("CardView::DrawBack");
	
	BRect aViewBounds = this->frame;
	
	this->view->SetPenSize(1);
	
	BRect cardRect = aViewBounds;
	this->view->SetHighColor(128, 128, 255);
	this->view->FillRoundRect(cardRect, cCardRadius, cCardRadius);
	
	cardRect.InsetBy(1, 1);
	cardRect.OffsetBy(1, 1);
	this->view->SetHighColor(64, 64, 255);
	this->view->FillRoundRect(cardRect, cCardRadius, cCardRadius);
	
	// Draw the edges.
	this->view->SetHighColor(0, 0, 0);
	this->view->StrokeRoundRect(aViewBounds, cCardRadius, cCardRadius);
}


// Draw the card.

void CardView::Draw(BRect)
{
	DB_TRACE("CardView::Draw");
	
	// Set up drawing environment.
	this->view->SetPenSize(1);
		
	// Draw the card front or back.
	if (this->faceUp)
		DrawFront();
	else
		DrawBack();
}


// Return the frame rectangle of the view.

BRect CardView::Frame() const
{
	DB_TRACE("CardView::Frame");
	
	return frame;
}


// Set the frame rectangle of the view.

void CardView::SetFrame(BRect frame)
{
	DB_TRACE("CardView::SetFrame");
	
	this->frame = frame;
}


// Return the view's card model.

CardModel *CardView::Model() const
{
	DB_TRACE("CardView::Model");
	
	return this->card;
}


// Set the view's card model.

void CardView::SetModel(CardModel* card)
{
	DB_TRACE("CardView::SetModel");
	
	this->card = card;
}


// Return TRUE if the card is face-up.

bool CardView::FaceUp() const
{
	return this->faceUp;
}


// Set whether the card is face-up or face-down. If the new value
// is different from the old, the view is invalidated. If this card
// view has no card model, it is an error to attempt to change it
// to face-up.

void CardView::SetFaceUp(bool faceUp)
{
	DB_TRACE("CardView::SetFaceUp");
	
	if (this->faceUp != faceUp)
	{
		this->faceUp = faceUp;
		this->view->Invalidate(this->frame);
	}
}
