/*
	
	ColumnView.cpp
	
	by Chris Page
	
*/

#ifndef _WINDOW_H
#include <Window.h>
#endif

#ifndef _REGION_H
#include <Region.h>
#endif

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif

#ifndef DECK_VIEW_H
#include "DeckView.h"
#endif

#ifndef HAND_VIEW_H
#include "HandView.h"
#endif

#ifndef FOUNDATION_VIEW_H
#include "FoundationView.h"
#endif

#ifndef SOLITAIRE_VIEW_H
#include "SolitaireView.h"
#endif

#ifndef COLUMN_MODEL_H
#include "ColumnModel.h"
#endif

#ifndef COLUMN_VIEW_H
#include "ColumnView.h"
#endif


// Create the view.

ColumnView::ColumnView(BRect frame, SolitaireView *solitaireView,
					   ColumnModel *column)
			: BView(frame, NULL, B_FOLLOW_NONE, B_WILL_DRAW)
{
	DB_TRACE("ColumnView::ColumnView");
	
	this->solitaireView = solitaireView;
	this->column = column;
	this->hilited = FALSE;
	this->oldCardCount = -1;
	
	// Create card views.
	BRect cardRect;
	for (long i = 0; i < cCardCountAll; i++)
	{
		this->cardViews[i] = new CardView((BView*) this, cardRect);
	}
	ModelChanged();
}


// Delete card views.

ColumnView::~ColumnView()
{
	DB_TRACE("ColumnView::~ColumnView");
	
	for (long i = 0; i < cCardCountAll; i++)
		delete this->cardViews[i];
}


// Initialize the view drawing environment.

void ColumnView::AttachedToWindow()
{
	DB_TRACE("ColumnView::AttachedToWindow");
	
	// Set the background color to card-table felt green.
	SetViewColor(SolitaireView::cViewColorRed,
				 SolitaireView::cViewColorGreen,
				 SolitaireView::cViewColorBlue);
}


// Draw the column.

void ColumnView::Draw(BRect rect)
{
	DB_TRACE("ColumnView::Draw");
	
	// Draw the column base marker.
	BRect aRect = Bounds();
	aRect.right -= 1;
	aRect.bottom = aRect.top + CardView::cCardHeight + 4 - 1;
	
	SetPenSize(2);
	SetHighColor(255, 255, 255);
	StrokeRoundRect(aRect, CardView::cCardRadius,
		CardView::cCardRadius);
	
	// Draw the cards.
	for (long i = 0; i < this->column->Count(); i++)
	{
		this->cardViews[i]->Draw(rect);
	}
}


// Handle mouse downs.

void ColumnView::MouseDown(BPoint point)
{
	DB_TRACE("ColumnView::MouseDown");
	
	if (this->column->IsEmpty())
		return;
	
	// See if the mouse point is in a face-up card.
	CardView* cardView = NULL;
	CardModel* card = NULL;
	BRect cardRect;
	for (long i = this->column->Count() - 1; i >= 0; i--)
	{
		cardView = this->cardViews[i];
		cardRect = cardView->Frame();
		if (cardRect.Contains(point))
		{
			card = cardView->Model();
			break;
		}
	}
	
	if (card == NULL || !this->solitaireView->solitaire->IsMovable(card))
		return;
	
	// If it was a double-click, try to move the card automatically.
	long clicks = Window()->CurrentMessage()->FindLong("clicks");
	if (clicks == 2)
	{
		// First try to move the card to a foundation, then to a column.
		
		SolitaireModel *solitaire = this->solitaireView->solitaire;
		
		for (int i = 0; i < SolitaireModel::cFoundationCount; i++)
		{
			FoundationModel *foundation = solitaire->foundations[i];
			if (solitaire->IsMovable(card, foundation))
			{
				solitaire->Move(card, foundation);
				
				// Notify views.
				this->solitaireView->foundationViews[i]->ModelChanged();
				this->solitaireView->deckView->ModelChanged();
				this->ModelChanged();
				this->solitaireView->ModelChanged();
				
				return;
			}
		}
		
		for (int i = 0; i < SolitaireModel::cColumnCount; i++)
		{
			ColumnModel *column = solitaire->columns[i];
			if (solitaire->IsMovable(card, column))
			{
				solitaire->Move(card, column);
				
				// Notify views.
				this->solitaireView->columnViews[i]->ModelChanged();
				this->solitaireView->deckView->ModelChanged();
				this->ModelChanged();
				this->solitaireView->ModelChanged();
				
				return;
			}
		}
	}
	else
	{
		// Start dragging the card.
		BMessage* message = new BMessage('CARD');
		message->AddObject("card", (BObject*) card);
		message->AddObject("sourceColumnView", (BObject*) this);
		DragMessage(message, cardRect);
	}
}


// If the given message contains a card that can be dropped on this
// column, return the card.

CardModel *ColumnView::AcceptCardMessage(BMessage *message) const
{
	DB_TRACE("ColumnView::AcceptCardMessage");
	
	if (message != NULL && message->what == 'CARD')
	{
		CardModel *card = (CardModel*) message->FindObject("card");
		
		if (card != NULL)
		{
			if (this->solitaireView->solitaire->IsMovable(card,
														  this->column))
			{
				return card;
			}
		}
	}
	
	return NULL;
}


// When the mouse moves in or out of the column, check whether it is
// dragging a card and if it can be moved to this column, hilite.

void ColumnView::MouseMoved(BPoint point, ulong transit,
							BMessage *message)
{
	DB_TRACE("ColumnView::MouseMoved");
	
	if (transit == B_ENTERED_VIEW)
	{
		// Check whether the message contains a card that can be
		// droppped on the column.
		CardModel *card = AcceptCardMessage(message);
		if (card != NULL)
		{
			Hilite(TRUE);
		}
	}
	else if (transit == B_EXITED_VIEW)
	{
		// Unhilite the column.
		Hilite(FALSE);
	}
}


// Handle dropped cards.

bool ColumnView::MessageDropped(BMessage *message, BPoint point,
								BPoint offset)
{
	DB_TRACE("ColumnView::MessageDropped");
	
	// Check whether the message contains a card that can be
	// droppped on the column.
	CardModel *card = AcceptCardMessage(message);
	if (card != NULL)
	{
		// Move the card to this column.
		this->solitaireView->solitaire->Move(card, this->column);
		
		// Notify views.
		HandView *handView
			= (HandView*) message->FindObject("sourceHandView");
		if (handView != NULL)
			handView->ModelChanged();
		ColumnView *columnView
			= (ColumnView*) message->FindObject("sourceColumnView");
		if (columnView != NULL)
			columnView->ModelChanged();
		this->solitaireView->deckView->ModelChanged();
		ModelChanged();
		this->solitaireView->ModelChanged();
		
		return TRUE;
	}
	
	return FALSE;
}


// Set the hilite of the column and update the display appropriately.

void ColumnView::Hilite(bool hilited)
{
	DB_TRACE("ColumnView::Hilite");
	
	if (this->hilited == hilited)
		return;
	
	this->hilited = hilited;
	
	CardView *cardView = NULL;
	BRect cardRect;
	if (!this->column->IsEmpty())
	{
		cardView = this->cardViews[this->column->Count() - 1];
		cardRect = cardView->Frame();
	}
	else
	{
		cardRect = this->cardViews[0]->Frame();
	}
	
	if (hilited)
	{
		// Hilite the top card.
		SetDrawingMode(B_OP_SUBTRACT);
		SetHighColor(64, 64, 64);
		FillRoundRect(cardRect, CardView::cCardRadius,
					  CardView::cCardRadius);
		SetDrawingMode(B_OP_COPY);
		SetHighColor(0, 0, 0);
		
		this->hiliteRect = cardRect;
	}
	else
	{
		// Unhilite the card. If this is the same card that was hilited
		// (the frame rect is the same as the hilited rect), draw it
		// immediately, otherwise just invalidate it.
		if (cardView != NULL && cardRect == this->hiliteRect)
			cardView->Draw(cardRect);
		else
			Invalidate(this->hiliteRect);
	}
}


// The column model changed. Update card views for the cards in the
// column.

void ColumnView::ModelChanged()
{
	DB_TRACE("ColumnView::ModelChanged");
	
	// Reset the hilite.
	Hilite(FALSE);
	
	// Calculate the starting card rect.
	BRect cardRect = Bounds();
	cardRect.left += 2;
	cardRect.top += 2;
	cardRect.right = cardRect.left + CardView::cCardWidth;
	cardRect.bottom = cardRect.top + CardView::cCardHeight;
	
	// Used to index through the card views. This is used by both
	// of the following for loops.
	long viewIndex = 0;
	
	// Update card views for each face-down card in the column.
	for (long i = 0; i < this->column->CountFaceDown(); i++)
	{
		CardView *cardView = this->cardViews[viewIndex++];
		
		if (cardView->Frame() != cardRect || cardView->FaceUp())
		{
			Invalidate(cardRect);
			Invalidate(cardView->Frame());
			
			cardView->SetFrame(cardRect);
			cardView->SetFaceUp(FALSE);
		}
		cardRect.OffsetBy(0, cCardFaceDownOffset);
	}
	
	// Update card views for each face-down card in the column.
	for (long i = 0; i < this->column->CountFaceUp(); i++)
	{
		CardView *cardView = this->cardViews[viewIndex++];
		CardModel *card
			= this->column->CardFaceUpAt(this->column->CountFaceUp()
										 - 1 - i);
		
		if (cardView->Frame() != cardRect || !cardView->FaceUp()
			|| cardView->Model() != card)
		{
			Invalidate(cardRect);
			Invalidate(cardView->Frame());
			
			cardView->SetFrame(cardRect);
			cardView->SetFaceUp(TRUE);
			cardView->SetModel(card);
		}
		cardRect.OffsetBy(0, cCardFaceUpOffset);
	}
	
	// If the number of cards has changed, update the column display.
	long cardCount = this->column->Count();
	if (this->oldCardCount != cardCount)
	{
		if (this->oldCardCount < cardCount)
			for (long i = this->oldCardCount; i < cardCount; i++)
			{
				Invalidate(this->cardViews[i]->Frame());
			}
		else
			for (long i = cardCount; i < this->oldCardCount; i++)
			{
				Invalidate(this->cardViews[i]->Frame());
			}
		this->oldCardCount = cardCount;
	}
	
	// Reset the hilite value.
//	this->hilited = FALSE;
}
