/*
	
	DeckView.cpp
	
	by Chris Page
	
*/

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif

#ifndef HAND_VIEW_H
#include "HandView.h"
#endif

#ifndef SOLITAIRE_MODEL_H
#include "SolitaireModel.h"
#endif

#ifndef SOLITAIRE_VIEW_H
#include "SolitaireView.h"
#endif

#ifndef DECK_MODEL_H
#include "DeckModel.h"
#endif

#ifndef DECK_VIEW_H
#include "DeckView.h"
#endif


// Create the view. Create child card views to draw the top of the deck.

DeckView::DeckView(BRect frame, SolitaireView *solitaireView,
				   DeckModel *deck)
	: BView(frame, NULL, B_FOLLOW_LEFT + B_FOLLOW_BOTTOM, B_WILL_DRAW)
{
	DB_TRACE("DeckView::DeckView");
	
	this->solitaireView = solitaireView;
	this->deck = deck;
	this->hilited = FALSE;
	this->oldDisplayedCardCount = -1;
	this->oldIsEmpty = FALSE;
	
	// Create card views to draw the cards of the deck. These card
	// views do not have a card model: they simply exist to draw the
	// back of the cards.
	BRect deckRect(Bounds());
	deckRect.right -= 2;
	deckRect.bottom -= 2;
	deckRect.left = deckRect.right - CardView::cCardWidth;
	deckRect.top = deckRect.bottom - CardView::cCardHeight;
	for (long i = 0; i < cCardViewCount; i++)
	{
		CardView* cardView = new CardView((BView*) this, deckRect);
		this->cardViews[i] = cardView;
		deckRect.OffsetBy(-2, -2);
	}
	
	ModelChanged();
}


// Delete card views.

DeckView::~DeckView()
{
	DB_TRACE("DeckView::~DeckView");
	
	for (long i = 0; i < cCardViewCount; i++)
		delete this->cardViews[i];
}


// Initialize the view drawing environment.

void DeckView::AttachedToWindow()
{
	DB_TRACE("DeckView::AttachedToWindow");
	
	// Set the background color to card-table felt green.
	SetViewColor(SolitaireView::cViewColorRed,
				 SolitaireView::cViewColorGreen,
				 SolitaireView::cViewColorBlue);
}


// Draw the deck.

void DeckView::Draw(BRect updateRect)
{
	DB_TRACE("DeckView::Draw");
	
	// Draw the deck placement marker.
	BRect aRect(Bounds());
	
	aRect.right -= 1;
	aRect.bottom -= 1;
	
	SetPenSize(2);
	SetHighColor(255, 255, 255);
	StrokeRoundRect(aRect, CardView::cCardRadius,
					CardView::cCardRadius);
	
	if (!this->deck->IsCardsEmpty())
	{
		// Draw the cards.
		long displayedCardCount = this->deck->CountCards();
		if (displayedCardCount > cCardViewCount)
			displayedCardCount = cCardViewCount;
		for (long i = 0; i < displayedCardCount; i++)
			this->cardViews[i]->Draw(updateRect);
	}
	else if (!this->deck->IsEmpty())
	{
		aRect.InsetBy(aRect.Width() / 4, aRect.Height() / 4);
		FillRoundRect(aRect, CardView::cCardRadius,
					  CardView::cCardRadius);
	}
}


// Handle mouse downs.

void DeckView::MouseDown(BPoint point)
{
	DB_TRACE("DeckView::MouseDown");
	
	if (this->deck->IsEmpty())
		return;
	
	// Get the top card view. If the deck is empty, just use the
	// the bottom card view.
	CardView *cardView;
	if (!this->deck->IsCardsEmpty())
	{
		long cardIndex = this->deck->CountCards();
		if (cardIndex > cCardViewCount)
			cardIndex = cCardViewCount;
		cardView = this->cardViews[cardIndex - 1];
	}
	else
	{
		cardView = this->cardViews[0];
	}
	BRect cardRect = cardView->Frame();
	
	// Track the mouse and hilite/unhilite the deck until the mouse
	// button is released.
	BPoint aMousePoint;
	ulong aButtons;
	do
	{
		snooze(20 * 1000);
		
		GetMouse(&aMousePoint, &aButtons, TRUE);
		
		Hilite(cardRect.Contains(aMousePoint));
		
	} while (aButtons);
	
	Hilite(FALSE);
	
	// If the deck was clicked, deal cards from the deck into the hand.
	if (cardRect.Contains(aMousePoint))
	{
		this->solitaireView->solitaire->DealHand();
		
		// Notify views.
		this->solitaireView->handView->ModelChanged();
		ModelChanged();
		this->solitaireView->ModelChanged();
	}
}


// Set the hilite of the deck and update the display appropriately.

void DeckView::Hilite(bool hilited)
{
	DB_TRACE("DeckView::Hilite");
	
	if (this->hilited == hilited)
		return;
	
	this->hilited = hilited;
	
	// Get the top card view. If the deck is empty, just use the
	// bottom card view.
	CardView *cardView = NULL;
	{
		if (!this->deck->IsCardsEmpty())
		{
			long cardIndex = this->deck->CountCards();
			if (cardIndex > cCardViewCount)
				cardIndex = cCardViewCount;
			cardView = this->cardViews[cardIndex - 1];
		}
		else
		{
			cardView = this->cardViews[0];
		}
	}
	BRect cardRect = cardView->Frame();
	
	if (hilited)
	{
		// Hilite the top card.
		SetDrawingMode(B_OP_SUBTRACT);
		SetHighColor(64, 64, 64);
		FillRoundRect(cardRect, CardView::cCardRadius,
					  CardView::cCardRadius);
		SetDrawingMode(B_OP_COPY);
		SetHighColor(0, 0, 0);
	}
	else
	{
		// Unhilite the top card.
		if (this->deck->IsCardsEmpty())
		{
			rgb_color saveColor = HighColor();
			
			SetHighColor(ViewColor());
			FillRoundRect(cardRect, CardView::cCardRadius,
						  CardView::cCardRadius);
			
			if (!this->deck->IsEmpty())
			{
				SetPenSize(2);
				SetHighColor(255, 255, 255);
				BRect aRect(Bounds());
				aRect.InsetBy(aRect.Width() / 4, aRect.Height() / 4);
				FillRoundRect(aRect, CardView::cCardRadius,
							  CardView::cCardRadius);
				SetPenSize(1);
			}
			
			SetHighColor(saveColor);
		}
		else
			cardView->Draw(cardRect);
	}
}


// The deck model changed. Hide/show card views to reflect the number of
// cards in the deck.

void DeckView::ModelChanged()
{
	DB_TRACE("DeckView::ModelChanged");
	
	// Only update the view if one of the following is true:
	//
	//		- The deck has become empty.
	//		- The number of displayed card views has changed.
	
	long displayedCardCount = this->deck->CountCards();
	if (displayedCardCount > cCardViewCount)
		displayedCardCount = cCardViewCount;
	
	if (this->oldDisplayedCardCount == displayedCardCount
		&& this->oldIsEmpty == this->deck->IsEmpty())
		return;
	
	this->oldDisplayedCardCount = displayedCardCount;
	this->oldIsEmpty = this->deck->IsEmpty();
	
	// Reset the hilite value.
	this->hilited = FALSE;
	
	// Invalidate the card views.
	for (long i = 0; i < cCardViewCount; i++)
	{
		Invalidate(this->cardViews[i]->Frame());
	}
}
