/*
	
	FoundationView.cpp
	
	by Chris Page
	
*/

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif

#ifndef DECK_VIEW_H
#include "DeckView.h"
#endif

#ifndef HAND_VIEW_H
#include "HandView.h"
#endif

#ifndef COLUMN_VIEW_H
#include "ColumnView.h"
#endif

#ifndef SOLITAIRE_VIEW_H
#include "SolitaireView.h"
#endif

#ifndef FOUNDATION_MODEL_H
#include "FoundationModel.h"
#endif

#ifndef FOUNDATION_VIEW_H
#include "FoundationView.h"
#endif


// Create the view. Create child card views to draw the top of the
// foundation.

FoundationView::FoundationView(BRect frame, SolitaireView *solitaireView,
							   FoundationModel* foundation)
	   	   : BView(frame, NULL, B_FOLLOW_NONE, B_WILL_DRAW)
{
	DB_TRACE("FoundationView::FoundationView");
	
	this->solitaireView = solitaireView;
	this->foundation = foundation;
	this->hilited = FALSE;
	
	// Create card views to draw the top cards of the deck.
	BRect foundationRect = Bounds();
	foundationRect.right -= 2;
	foundationRect.bottom -= 2;
	foundationRect.left = foundationRect.right - CardView::cCardWidth;
	foundationRect.top = foundationRect.bottom - CardView::cCardHeight;
	for (long i = 0; i < cCardViewCount; i++)
	{
		this->cardViews[i] = new CardView((BView*) this, foundationRect);
		foundationRect.OffsetBy(-2, -2);
	}
	
	ModelChanged();
}


// Delete card views.

FoundationView::~FoundationView()
{
	DB_TRACE("FoundationView::~FoundationView");
	
	for (long i = 0; i < cCardViewCount; i++)
		delete this->cardViews[i];
}


// Initialize the view drawing environment.

void FoundationView::AttachedToWindow()
{
	DB_TRACE("FoundationView::AttachedToWindow");
	
	// Set the background color to card-table felt green.
	SetViewColor(SolitaireView::cViewColorRed,
				 SolitaireView::cViewColorGreen,
				 SolitaireView::cViewColorBlue);
}


// Draw the foundation.

void FoundationView::Draw(BRect rect)
{
	DB_TRACE("FoundationView::Draw");
	
	// Draw the foundation placement marker.
	BRect aRect = Bounds();
	aRect.right -= 1;
	aRect.bottom -= 1;
	
	SetPenSize(2);
	SetHighColor(255, 255, 255);
	StrokeRoundRect(aRect, CardView::cCardRadius,
		CardView::cCardRadius);
	
	// Draw the cards.
	long cardViewCount = this->foundation->Count();
	if (cardViewCount > cCardViewCount)
		cardViewCount = cCardViewCount;
	for (long i = 0; i < cardViewCount; i++)
	{
		this->cardViews[i]->Draw(rect);
	}
}


// If the given message contains a card that can be dropped on this
// column, return the card.

CardModel *FoundationView::AcceptCardMessage(BMessage *message) const
{
	DB_TRACE("FoundationView::AcceptCardMessage");
	
	if (message != NULL && message->what == 'CARD')
	{
		CardModel *card = (CardModel*) message->FindObject("card");
		
		if (card != NULL)
		{
			if (this->solitaireView->solitaire->
				IsMovable(card, this->foundation))
			{
				return card;
			}
		}
	}
	
	return NULL;
}


// When the mouse moves in or out of the foundation, check whether it is
// dragging a card and if it can be moved to this foundation, hilite.

void FoundationView::MouseMoved(BPoint point, ulong transit,
								BMessage *message)
{
	DB_TRACE("FoundationView::MouseMoved");
	
	if (transit == B_ENTERED_VIEW)
	{
		// Check whether the message contains a card that can be
		// droppped on the foundation.
		CardModel *card = AcceptCardMessage(message);
		if (card != NULL)
		{
			Hilite(TRUE);
		}
	}
	else if (transit == B_EXITED_VIEW)
	{
		// Unhilite the column.
		Hilite(FALSE);
	}
}


// Handle dropped cards.

bool FoundationView::MessageDropped(BMessage *message, BPoint point,
									BPoint offset)
{
	DB_TRACE("FoundationView::MessageDropped");
	
	// Check whether the message contains a card that can be
	// droppped on the foundation.
	CardModel *card = AcceptCardMessage(message);
	if (card != NULL)
	{
		// Move the card to this foundation.
		this->solitaireView->solitaire->Move(card, this->foundation);
		
		// Notify views.
		HandView *handView
			= (HandView*) message->FindObject("sourceHandView");
		if (handView != NULL)
			handView->ModelChanged();
		ColumnView *columnView
			= (ColumnView*) message->FindObject("sourceColumnView");
		if (columnView != NULL)
			columnView->ModelChanged();
		this->solitaireView->deckView->ModelChanged();
		ModelChanged();
		this->solitaireView->ModelChanged();
		
		return TRUE;
	}
	
	return FALSE;
}


// Set the hilite of the foundation and update the display appropriately.

void FoundationView::Hilite(bool hilited)
{
	DB_TRACE("FoundationView::Hilite");
	
	if (this->hilited == hilited)
		return;
	
	this->hilited = hilited;
	
	CardView *cardView = NULL;
	BRect cardRect;
	if (!this->foundation->IsEmpty())
	{
		long cardIndex = this->foundation->Count();
		if (cardIndex > cCardViewCount)
			cardIndex = cCardViewCount;
		cardView = this->cardViews[cardIndex - 1];
		cardRect = cardView->Frame();
	}
	else
	{
		cardRect = this->cardViews[0]->Frame();
	}
	
	if (hilited)
	{
		// Hilite the top card.
		SetDrawingMode(B_OP_SUBTRACT);
		SetHighColor(64, 64, 64);
		FillRoundRect(cardRect, CardView::cCardRadius,
					  CardView::cCardRadius);
		SetDrawingMode(B_OP_COPY);
		SetHighColor(0, 0, 0);
	}
	else
	{
		if (cardView != NULL)
			cardView->Draw(cardRect);
		else
			Invalidate(cardRect);
	}
}


// The foundation model changed. Update card views for the cards in the
// foundation.

void FoundationView::ModelChanged()
{
	DB_TRACE("FoundationView::ModelChanged");
	
	// Reset the hilite.
	Hilite(FALSE);
	
	// Set the model of the card views.
	long cardCount = this->foundation->Count();
	long cardViewCount = cardCount;
	if (cardViewCount > cCardViewCount)
		cardViewCount = cCardViewCount;
	for (long i = 0; i < cardViewCount; i++)
	{
		CardModel* card
			= this->foundation->CardAt(cardCount - cardViewCount + i);
		CardView* cardView = this->cardViews[i];
		cardView->SetModel(card);
		cardView->SetFaceUp(TRUE);
	}
	
	// Invalidate the top card view. Only the top view is updated
	// because we guarantee that only the edge of the lower cards
	// will be visible (i.e. none of the card face will be visible).
	if (cardViewCount > 0)
		Invalidate(this->cardViews[cardViewCount - 1]->Frame());
}
