/*
	
	HandView.cpp
	
	by Chris Page
	
*/

#ifndef _WINDOW_H
#include <Window.h>
#endif

#ifndef CARD_MODEL_H
#include "CardModel.h"
#endif

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif

#ifndef DECK_VIEW_H
#include "DeckView.h"
#endif

#ifndef COLUMN_VIEW_H
#include "ColumnView.h"
#endif

#ifndef FOUNDATION_VIEW_H
#include "FoundationView.h"
#endif

#ifndef SOLITAIRE_MODEL_H
#include "SolitaireModel.h"
#endif

#ifndef SOLITAIRE_VIEW_H
#include "SolitaireView.h"
#endif

#ifndef HAND_MODEL_H
#include "HandModel.h"
#endif

#ifndef HAND_VIEW_H
#include "HandView.h"
#endif


// Create the view.

HandView::HandView(BRect frame, SolitaireView *solitaireView,
				   HandModel *hand)
	: BView(frame, NULL, B_FOLLOW_LEFT + B_FOLLOW_BOTTOM, B_WILL_DRAW)
{
	DB_TRACE("HandView::HandView");
	
	this->solitaireView = solitaireView;
	this->hand = hand;
	this->hilited = FALSE;
	
	// Create card views to draw the top cards of the hand.
	BRect handRect = Bounds();
	handRect.right = handRect.left + CardView::cCardWidth;
	for (long i = 0; i < HandModel::cCardCount; i++)
	{
		CardView* cardView = new CardView((BView*) this, handRect);
		this->cardViews[i] = cardView;
		handRect.OffsetBy(cCardOffset, 0);
	}
	
	ModelChanged();
}


// Delete card views.

HandView::~HandView()
{
	DB_TRACE("HandView::~HandView");
	
	for (long i = 0; i < HandModel::cCardCount; i++)
		delete this->cardViews[i];
}


// Initialize the view drawing environment.

void HandView::AttachedToWindow()
{
	DB_TRACE("HandView::AttachedToWindow");
	
	// Set the background color to card-table felt green.
	SetViewColor(SolitaireView::cViewColorRed,
				 SolitaireView::cViewColorGreen,
				 SolitaireView::cViewColorBlue);
}


// Draw the hand.

void HandView::Draw(BRect rect)
{
	DB_TRACE("HandView::Draw");
	
	// Draw the cards.
	for (long i = 0; i < this->hand->Count(); i++)
		this->cardViews[i]->Draw(rect);
}


// Handle mouse downs.

void HandView::MouseDown(BPoint point)
{
	DB_TRACE("HandView::MouseDown");
	
	if (this->hand->IsEmpty())
		return;
	
	CardView* cardView = this->cardViews[this->hand->Count() - 1];
	CardModel* card = cardView->Model();
	
	// Check to see if the top card was clicked.
	BRect cardRect = cardView->Frame();
	if (!cardRect.Contains(point))
		return;
	
	// If it was a double-click, try to move the card automatically.
	long clicks = Window()->CurrentMessage()->FindLong("clicks");
	if (clicks == 2)
	{
		// If it is an Ace, move it to an empty foundation, otherwise,
		// first try to move it to a column, then to a foundation.
		
		SolitaireModel *solitaire = this->solitaireView->solitaire;
		
		if (card->rank != 1)
		{
			for (int i = 0; i < SolitaireModel::cColumnCount; i++)
			{
				ColumnModel *column = solitaire->columns[i];
				if (solitaire->IsMovable(card, column))
				{
					solitaire->Move(card, column);
					this->solitaireView->columnViews[i]->ModelChanged();
					this->solitaireView->deckView->ModelChanged();
					this->ModelChanged();
					this->solitaireView->ModelChanged();
					
					return;
				}
			}
		}
		
		for (int i = 0; i < SolitaireModel::cFoundationCount; i++)
		{
			FoundationModel *foundation = solitaire->foundations[i];
			if (solitaire->IsMovable(card, foundation))
			{
				solitaire->Move(card, foundation);
				this->solitaireView->foundationViews[i]->ModelChanged();
				this->solitaireView->deckView->ModelChanged();
				this->ModelChanged();
				this->solitaireView->ModelChanged();
				
				return;
			}
		}
	}
	else
	{
		// Start dragging the top card.
		BMessage* message = new BMessage('CARD');
		message->AddObject("card", (BObject*) card);
		message->AddObject("sourceHandView", (BObject*) this);
		DragMessage(message, cardRect);
	}
}


// NOTE: The hiliting functions are no longer used, but I've left
// them in for now; I may decide to reinstate drag source hiliting.

// Set the hilite of the hand and update the display appropriately.

void HandView::Hilite(bool hilited)
{
	DB_TRACE("HandView::Hilite");
	
	if (this->hilited == hilited)
		return;
	
	this->hilited = hilited;
	
	CardView *cardView;
	if (this->hand->IsEmpty())
		cardView = this->cardViews[0];
	else
		cardView = this->cardViews[this->hand->Count() - 1];
	BRect cardRect = cardView->Frame();
	
	if (hilited)
	{
		// Hilite the top card.
		SetDrawingMode(B_OP_SUBTRACT);
		SetHighColor(64, 64, 64);
		FillRoundRect(cardRect, CardView::cCardRadius,
					  CardView::cCardRadius);
		SetDrawingMode(B_OP_COPY);
		SetHighColor(0, 0, 0);
	}
	else
		cardView->Draw(cardRect);
}


// The hand model changed. Update card views for the cards in the hand.

void HandView::ModelChanged()
{
	DB_TRACE("HandView::ModelChanged");
	
	// Set the model of the cards in the hand.
	for (long i = 0; i < this->hand->Count(); i++)
	{
		CardModel *card = this->hand->CardAt(i);
		CardView *cardView = this->cardViews[i];
		cardView->SetModel(card);
		cardView->SetFaceUp(TRUE);
	}
	
	// Reset the hilite value.
	this->hilited = FALSE;
	
	Invalidate();
}
