/*
	
	SolitaireView.cpp
	
	by Chris Page
	
*/

#ifndef _WINDOW_H
#include <Window.h>
#endif

#ifndef _ALERT_H
#include <Alert.h>
#endif

#ifndef SOLITAIRE_MODEL_H
#include "SolitaireModel.h"
#endif

#ifndef CARD_VIEW_H
#include "CardView.h"
#endif

#ifndef DECK_VIEW_H
#include "DeckView.h"
#endif

#ifndef HAND_VIEW_H
#include "HandView.h"
#endif

#ifndef COLUMN_MODEL_H
#include "ColumnModel.h"
#endif

#ifndef COLUMN_VIEW_H
#include "ColumnView.h"
#endif

#ifndef FOUNDATION_VIEW_H
#include "FoundationView.h"
#endif

#ifndef SOLITAIRE_VIEW_H
#include "SolitaireView.h"
#endif


// Create the view and its subviews.

SolitaireView::SolitaireView(BRect frame, SolitaireModel* solitaire)
	   	   : BView(frame, NULL, B_FOLLOW_ALL, B_WILL_DRAW)
{
	DB_TRACE("SolitaireView::SolitaireView");
	
	this->solitaire = solitaire;
	
	BRect deckRect(Bounds());
	deckRect.left += cMargin;
	deckRect.bottom -= cMargin;
	deckRect.right = deckRect.left
		+ (CardView::cCardWidth + 3 * 2 + 2);
	deckRect.top = deckRect.bottom
		- (CardView::cCardHeight + 3 * 2 + 2);
	this->deckView = new DeckView(deckRect, this, solitaire->deck);
	AddChild(this->deckView);
	
	BRect handRect(Bounds());
	handRect.left = deckRect.right + cHandOffset;
	handRect.bottom -= cMargin;
	handRect.right = handRect.left + CardView::cCardWidth
					 + 2 * HandView::cCardOffset;
	handRect.top = handRect.bottom - CardView::cCardHeight;
	this->handView = new HandView(handRect, this, solitaire->hand);
	AddChild(this->handView);
	
	BRect columnRect(Bounds());
	columnRect.left += cMargin;
	columnRect.top += cMargin + CardView::cCardHeight + 3 * 2 + 2
					  + cFoundationOffset;
	columnRect.bottom = columnRect.top
		+ ColumnView::cCardCount * ColumnView::cCardFaceUpOffset
		+ ColumnView::cCardFaceDownCount * ColumnView::cCardFaceDownOffset
		+ CardView::cCardHeight + 4;
	columnRect.right = columnRect.left + CardView::cCardWidth + 4;
	for (long i = 0; i < SolitaireModel::cColumnCount; i++)
	{
		this->columnViews[i]
			= new ColumnView(columnRect, this, solitaire->columns[i]);
		AddChild(this->columnViews[i]);
		columnRect.OffsetBy(CardView::cCardWidth + 4 + cColumnOffset, 0);
	}
	
	BRect foundationRect(Bounds());
	foundationRect.right
		= this->columnViews[SolitaireModel::cColumnCount - 1]
			->Frame().right;
	foundationRect.top += cMargin;
	foundationRect.bottom = foundationRect.top
							+ (CardView::cCardHeight + 3 * 2 + 2);
	foundationRect.left = foundationRect.right
						  - (CardView::cCardWidth + 3 * 2 + 2);
	for (long i = 0; i < SolitaireModel::cFoundationCount; i++)
	{
		this->foundationViews[i] = new FoundationView(foundationRect,
								   this, solitaire->foundations[i]);
		AddChild(this->foundationViews[i]);
		foundationRect.OffsetBy(
			-(CardView::cCardWidth + 3 * 2 + 2 + cColumnOffset), 0);
	}
}


// Initialize the view drawing environment.

void SolitaireView::AttachedToWindow()
{
	DB_TRACE("SolitaireView::AttachedToWindow");
	
	// Set the background color to card-table felt green.
	SetViewColor(cViewColorRed, cViewColorGreen, cViewColorBlue);
}


// Update the view to match the current model. If the game is in a
// winning state, play it out automatically.

void SolitaireView::ModelChanged()
{
	DB_TRACE("SolitaireView::ModelChanged");
	
	// Check for a won game.
	if (!this->solitaire->Won())
		return;
	
	// Fun sound.
	(void) beep();
	
	// Used to check whether all the columns are empty.
	bool movableCardsRemaining;
	int emptyColumnCount;
	
	// Move all the cards onto the foundations.
	do
	{
		movableCardsRemaining = FALSE;
		emptyColumnCount = 0;
		
		// Search for the first bottom column card that can
		// be moved onto a foundation, then move it.
		for (int i = 0; i < SolitaireModel::cColumnCount; i++)
		{
			ColumnModel *column = this->solitaire->columns[i];
			if (column->IsEmpty())
			{
				emptyColumnCount++;
				
				continue;
			}
			
			CardModel *card = column->Bottom();
			for (int j = 0; j < SolitaireModel::cFoundationCount; j++)
			{
				FoundationModel *foundation
					= this->solitaire->foundations[j];
				if (this->solitaire->IsMovable(card, foundation))
				{
					movableCardsRemaining = TRUE;
					this->solitaire->Move(card, foundation);
					this->columnViews[i]->ModelChanged();
					this->foundationViews[j]->ModelChanged();
					
					// Update the display and synchronize so the
					// winning sequence won't blur by.
					this->Window()->UpdateIfNeeded();
					Sync();
				}
			}
		}
	} while (movableCardsRemaining &&
			 emptyColumnCount < SolitaireModel::cColumnCount);
	
	// Fun sound.
	(void) beep();
	
	// Congratulatory message.
	BAlert *alert = new BAlert(NULL,
		"Congratulations! You won!",
		"Happy! Happy! Joy! Joy!", NULL, NULL,
		B_WIDTH_FROM_LABEL);
	alert->Go();
}
